<?php

require_once 'lib/utils.php';
require_once 'ext/TorrServer.php';

function hd_print($x) {
    echo("$x\n");
}

function hd_error_silencer($severity, $msg, $filename, $linenum)
{ }

function hd_silence_warnings() {
    error_reporting(0);
    set_error_handler("hd_error_silencer");
}

function hd_restore_warnings() {
    restore_error_handler();
    error_reporting(E_STRICT | E_ALL);
}

function usage() {
    hd_print("Usage: tscli.php <command> [<params>...]");
    hd_print("Examples:");
    hd_print("  tscli.php status");
    hd_print("  tscli.php start");
    hd_print("  tscli.php stop");
    hd_print("  tscli.php list_torrents");
    hd_print("  tscli.php settings");
    hd_print("  tscli.php set_setting <key> <value>");
    exit(1);
}

function fatal($msg) {
    hd_print("FATAL ERROR: $msg");
    exit(1);
}

function finish($msg) {
    hd_print("DONE: $msg");
    exit(0);
}

if (count($argv) < 2)
    usage();

$cmd = $argv[1];
if ($cmd == 'status') {
    run_status($argv);
} else if ($cmd == 'start') {
    run_start($argv);
} else if ($cmd == 'stop') {
    run_stop($argv);
} else if ($cmd == 'list_torrents') {
    run_list_torrents($argv);
} else if ($cmd == 'settings') {
    run_settings($argv);
} else if ($cmd == 'set_setting') {
    run_set_setting($argv);
} else {
    usage();
}

///////////////////////////////////////////////////////////////////////////

function run_status($argv) {
    $ok = TorrServer::getOnlineStatusServer();
    hd_print("TorrServer status: " . ($ok ? "running" : "not running"));
    exit($ok ? 0 : 1);
}

function run_start($argv) {

    if (TorrServer::getOnlineStatusServer())
        finish("TorrServer already started");

    clearstatcache();

    $dirpath = "/flashdata/ts";
    $binpath = "$dirpath/ts";
    $logpath = "/tmp/run/ts.log";
    $setspath = $dirpath;

    if (!is_dir($dirpath))
        fatal("dir $dirpath not exist");
    if (!is_file($binpath) || !is_executable($binpath))
        fatal("binary file $binpath not exist or invalid");

    putenv("GODEBUG=madvdontneed=1");

    $cmd = "$binpath -k -d $setspath -l $logpath 1>>$logpath 2>&1 &";
    hd_print("Running: $cmd");
    system($cmd, $ret);
    hd_print("Waiting for TorrServer start...");

    for ($i = 0; $i < 25; $i++)
    {
        if ($i > 0)
            hd_print("Still waiting...");

        usleep(400000);
        if (TorrServer::getOnlineStatusServer())
            finish("TorrServer started");
    }

    fatal("timed out waiting for TorrServer start");
}

function run_stop($argv) {
    if (!TorrServer::getOnlineStatusServer()) {
        hd_print("TorrServer not running");
        exit(0);
    }

    $status = TorrServer::torrserverActionStop();
    if (!$status)
        finish("TorrServer stopped");
    hd_print("TorrServer stop: $status");
    exit(1);
}

function run_list_torrents($argv) {
    if (!TorrServer::getOnlineStatusServer())
        fatal("TorrServer not running");

    $res = TorrServer::torrserverActionListTorrents();
    if ($res === false)
        fatal("request failed");
    hd_print(pretty_json_encode($res));
    exit(0);
}

function run_settings($argv) {
    if (!TorrServer::getOnlineStatusServer())
        fatal("TorrServer not running");

    $res = TorrServer::torrserverActionSettingsRead();
    if ($res === false)
        fatal("request failed");
    hd_print(pretty_json_encode($res));
    exit(0);
}

function run_set_setting($argv) {
    if (count($argv) != 4)
        usage();
    if (!TorrServer::getOnlineStatusServer())
        fatal("TorrServer not running");

    $res = TorrServer::torrserverActionSettingsRead();
    if ($res === false)
        fatal("request failed");
    hd_print(pretty_json_encode($res));

    $key = $argv[2];
    $val = (string) $argv[3];
    if ($val == 'true')
        $val = true;
    else if ($val == 'false')
        $val = false;
    else if (is_numeric($val))
        $val = (int) $val;
    $res->$key = $val;

    $res = TorrServer::torrserverActionSettingsWrite((array) $res);
    if ($res === false)
        fatal("request failed");
    hd_print(pretty_json_encode($res));

    exit(0);
}

///////////////////////////////////////////////////////////////////////////

function pretty_print( $json )
{
    $result = '';
    $level = 0;
    $in_quotes = false;
    $in_escape = false;
    $ends_line_level = NULL;
    $json_length = strlen( $json );

    for( $i = 0; $i < $json_length; $i++ ) {
        $char = $json[$i];
        $new_line_level = NULL;
        $post = "";
        if( $ends_line_level !== NULL ) {
            $new_line_level = $ends_line_level;
            $ends_line_level = NULL;
        }
        if ( $in_escape ) {
            $in_escape = false;
        } else if( $char === '"' ) {
            $in_quotes = !$in_quotes;
        } else if( ! $in_quotes ) {
            switch( $char ) {
                case '}': case ']':
                    $level--;
                    $ends_line_level = NULL;
                    $new_line_level = $level;
                    break;

                case '{': case '[':
                    $level++;
                case ',':
                    $ends_line_level = $level;
                    break;

                case ':':
                    $post = " ";
                    break;

                case " ": case "\t": case "\n": case "\r":
                    $char = "";
                    $ends_line_level = $new_line_level;
                    $new_line_level = NULL;
                    break;
            }
        } else if ( $char === '\\' ) {
            $in_escape = true;
        }
        if( $new_line_level !== NULL ) {
            $result .= "\n".str_repeat( "  ", $new_line_level );
        }
        $result .= $char.$post;
    }

    return $result;
}

function pretty_json_encode($obj)
{
    $pattern = "/\\\\u([0-9a-fA-F]{4})/";
    $callback = function ($m) {
        return html_entity_decode("&#x$m[1];", ENT_QUOTES, 'UTF-8');
    };
    return pretty_print(
        str_replace('\\/', '/',
            preg_replace_callback($pattern, $callback,
                json_encode($obj))));
}

?>
